-----------------------------------------------------------------------------------
-- Company: 
-- Engineer: ici et la
-- 
-- Create Date:    17:32:32 07/03/2022 
-- Design Name: 
-- Module Name:    FSM - Behavioral 
-- Project Name:  LCD_BT471
-- 
-- Revision: 2 	05/10/2022
-- Reduce of Load signal time from 120ns to 40ns to avoid bug when RSX change in less than 100nS after WR_bar rise up

-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all ;

entity FSM is
    Port ( RS0 : in  STD_LOGIC;
           RS1 : in  STD_LOGIC;
			  RS2	: in	STD_LOGIC;
           WR_bar : in  STD_LOGIC;
           clk : in  STD_LOGIC;
           reset : in  STD_LOGIC;
           Load_Pix_Reg : out  STD_LOGIC;
           Load_Add_Reg : out  STD_LOGIC;
           Load_Rouge_Reg : out  STD_LOGIC;
           Load_vert_Reg : out  STD_LOGIC;
           Load_Bleu_Reg : out  STD_LOGIC;
			  Set_Mux		: out STD_LOGIC;
			  M_RS2			: out STD_LOGIC;
           WR_bar_Ram : out  STD_LOGIC;
			  current_state, next_state : inout STD_LOGIC_VECTOR (3 downto 0)
		);
end FSM;

architecture Behavioral of FSM is
	signal count 	: STD_LOGIC := '0';	 -- counter to count number of clock cycle length needed to build write signal in register and Video RAM ( one pulse in this case)
	signal on_count : STD_LOGIC := '0';	 -- Flag to pass next step only when the write pulse is end
	signal on_process: STD_LOGIC := '0'; -- Flag for M_RS2 during write process color or palette
begin

	P_STATE : process(clk, Reset)

	begin
		if reset = '1' then
				current_state <= "0000";
				count <= '0';				
		elsif (clk'event and clk = '1') then
			current_state <= next_state;
			if on_count = '1' then				-- counter and flag for loading signals
				count <= '1';					-- This gives a time pulse of 1 cycle : 40nS
			else
				count <= '0';				
			end if;
		end if;
	end process P_state;
	

	
-- Event management on the rising edge of WR_bar

	P_NEXT_STATE : process(WR_bar, RS0,RS1,RS2,current_state, on_count, count)

	begin
	
	on_count <= '0';

	
	case current_state is
	

	
--------------------------------------------------------------
-- Pixel mask register wrinting case RS0 = '0' and RS1 = '1' 
--------------------------------------------------------------	
	-- waiting for Low state detection of WR_bar
		when "0000" =>

			next_state <= "0000";
			if (WR_bar = '0') then
				if( RS0 = '0' and RS1 = '1' ) then 		-- Case Pixel Mask Register
					next_state <= "0001";
				elsif (RS0 = '0' and RS1 = '0' ) then  	-- Case Address register
					next_state <= "0010"	;
				end if; 
			else
				next_state <= "0000";
				
			end if;
	
	-- Load Pixel Mask register
		when "0001" =>
			next_state <= "0001";
	
			if (WR_bar = '1') then
						if count = '1' then
							next_state <= "0000";
						else							
							on_count <= '1';
							next_state <= "0001";
						end if;
			end if;
------------------------------------------------------------			
-- Address Register writing case	RS0 = '0' and RS1 = '0'	
------------------------------------------------------------
	-- waiting for high state detection of WR_bar
	
		when "0010" =>
				next_state <= "0010";
				if (WR_bar = '1') then
							if count = '1' then
								next_state <= "0011"; 	-- pulse sent
							else
								on_count <= '1'; 		-- start to send pulse							
								next_state <= "0010"; 	-- wait until the end of pulse sending			
							end if;
				end if;
				
------------------------------------------------------------			
-- Red register writing case
------------------------------------------------------------	
	-- waiting for Low state detection of WR_bar
	
		when "0011" =>
				next_state <= "0011";
				if (WR_bar = '0') then
					if (RS0 = '1' and RS1 = '0' ) then 
						next_state <= "0100";
					else 
						next_state <= "0011";
					end if;
				end if;
				
	-- waiting for High state detection of WR_bar
	
		when "0100" =>	
			next_state <= "0100";

			if (WR_bar = '1') then
						if count = '1' then
							next_state <= "0101";
						else
							on_count <= '1';
							next_state <= "0100";
						end if;
			end if;
			
			
----------------------------------------------------------			
-- Green register writing case
----------------------------------------------------------
	-- waiting for Low state detection of WR_bar
	
		when "0101" =>
				next_state <= "0101";
				if ( WR_bar = '0') then 
					if( RS0 = '1' and RS1 = '0' ) then
						next_state <= "0110";
					else 
						next_state <= "0101";
					end if;
				end if;
				
	-- waiting for High state detection of WR_bar
	
		when "0110" =>	
				next_state <= "0110";
		
				if (WR_bar = '1') then
						if count = '1' then
							next_state <= "0111";
						else					
							on_count <= '1';
							next_state <= "0110";
						end if;
				end if;
				
			
	
-----------------------------------------------------------	
-- Blue register wrinting case
-----------------------------------------------------------
	-- waiting for Low state detection of WR_bar
	
		when "0111" =>
				next_state <= "0111";
				if ( WR_bar = '0') then
					if(RS0 = '1' and RS1 = '0' ) then
						next_state <= "1000";
					else 
						next_state <= "0100";
					end if;
				end if;
				
	-- waiting for High state detection of WR_bar	
	
		when "1000" =>	
				next_state <= "1000";	
				if (WR_bar = '1') then				
						if count = '1' then
							next_state <= "1001";
						else	
							on_count <= '1';
							next_state <= "1000";
						end if;				
				
				end if;	
				

----------------------------------------------------------				
-- Load the color in the RAM
----------------------------------------------------------
		when "1001" =>	
				
						if count = '1' then
							next_state <= "0000";
						else				
							on_count <= '1';
							next_state <= "1001";
						end if;
						
								
		when others => next_state <= "0000";

		end case;	
		


		end process P_NEXT_STATE;
-----------------------------------------------------------------		
--	Management of the RS2 Memory for pallet or color RAM address
-----------------------------------------------------------------
 	
	P_M_RS2 : process( clk, reset, RS2 ,WR_bar, current_state)
			begin
			if rising_edge(clk) then
				if (reset = '1' or current_state = "0000") then 
					M_RS2 <= '0';
					on_process <= '0';
				elsif (WR_bar = '0' and current_state = "0010" and on_process ='0') then
					M_RS2 <= RS2;
					on_process <= '1';
				end if;
			end if;
			end process P_M_RS2;
		
-----------------------------------------------------------
-- Mangement of the outputs pulses
-----------------------------------------------------------
		

		
	Load_Pix_Reg <= '1' when current_state = "0001" and count /= '0' else '0';
	Load_Add_Reg <= '1'  when current_state = "0010" and count /= '0' else '0';
	Load_Rouge_Reg <= '1' when current_state = "0100" and count /= '0' else '0';
	Load_Vert_Reg <= '1' when current_state = "0110" and count /= '0' else '0';
	Load_bleu_Reg <= '1' when current_state = "1000" and count /= '0' else '0';
	WR_bar_Ram  <= '0' when current_state = "1001"  and count /= '0' else '1';	
	Set_MUX		<= '0' when current_state = "1001" else '1';


	
	
end Behavioral;

