library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
-- library UNISIM;
-- use UNISIM.VComponents.all;

entity RAM_DAC is
    Port ( Palette : in  STD_LOGIC_VECTOR (4 downto 0);
           Overlay : in  STD_LOGIC_VECTOR (3 downto 0);
			  Data : in STD_LOGIC_VECTOR (7 downto 0);
           RS2,RS1,RS0 : in  STD_LOGIC;
			  WR_bar: STD_LOGIC;
           clk : in  STD_LOGIC;
			  --reset : in STD_LOGIC;  --Internally generated
			  blank_bar : in STD_LOGIC;
			  --LCD_clock : out STD_LOGIC;  -- Not used in V2 LCD
			  --LCD_blank_bar : out STD_LOGIC;  --Not used in V2 LCD
           LCD_Out1, LCD_Out2, LCD_Out3 : out STD_LOGIC;  -- outputs to lvds converters to drive LCD inputs
           LCD_Clock_Out : out STD_LOGIC;  --  LVDS clock to LCD
           BL : in STD_LOGIC_VECTOR (2 downto 0);  --  inputs to set lcd brightness (PWM)
           PWM : out STD_LOGIC;  -- PWM signal for LCD backlight
           LED_ON : out STD_LOGIC; -- LCD Backlight on/off control
           led1 : buffer STD_LOGIC;
           --Rouge_LCD,Vert_LCD,Bleu_LCD : out  STD_LOGIC_VECTOR (7 downto 0);  --Not used in V2 LCD.  Becomes a signal instead.
			  current_state : inout STD_LOGIC_VECTOR ( 3 downto 0);
			  --rajout pour Test bench
			  RAM_ADDR_bench: out STD_LOGIC_VECTOR (7 downto 0); --**************************** pour voir les adresses dans le TB***************
			  Debug_in : in STD_LOGIC_VECTOR (3 downto 0);
			  Debug_out : out STD_LOGIC_VECTOR (3 downto 0)
	);
		
end RAM_DAC;

architecture Behavioral of RAM_DAC is
-- Description des composants

component clk_wiz_0
        port (
          clk_out1          : out    std_logic;
          clk_out2          : out    std_logic;
          locked            : out    std_logic;
          clk_in1           : in     std_logic
     );
    end component;
    
    component reset_generator 
     port (
        clock : in std_logic;
        reset : out std_logic
        );
    end component;

	component REG_8 is
		port(   clk,raz,load    :   in std_logic;
						d_in        :   in std_logic_vector(7 downto 0);
						d_out       :   out std_logic_vector(7 downto 0)
	 );	
	end component;
	
	component PIX_MASK is
		Port (	IN1 : in  STD_LOGIC_VECTOR (7 downto 0);
					IN2 : in  STD_LOGIC_VECTOR (7 downto 0);
				SORTIE : out  STD_LOGIC_VECTOR (7 downto 0)
	);
	end component;
	
	component MUX_4 is
		Port (	E0 : in  STD_LOGIC_VECTOR (7 downto 0); 
					E1 : in  STD_LOGIC_VECTOR (7 downto 0); 
					E2 : in  STD_LOGIC_VECTOR (7 downto 0); 
					E3 : in  STD_LOGIC_VECTOR (7 downto 0);
				  Sel : in  STD_LOGIC_VECTOR (1 downto 0);			  
			  Sortie : out  STD_LOGIC_VECTOR (7 downto 0)
	);
	end component;
	
	component RAM_VIDEO is
		port(
				RAM_ADDR 	: in std_logic_vector(5 downto 0);	-- Address to write/read RAM
				RAM_DATA_IN	: in std_logic_vector(7 downto 0);	-- Data to write into RAM
				RAM_WR		: in std_logic; 					-- Write enable 
				RAM_clk		: in std_logic; 					-- clk input for RAM
				RAM_DATA_OUT: out std_logic_vector(7 downto 0) 	-- Data output of RAM
	);
	end component;
	
	component Serializer is
	   port(
	           CLK_25 : in std_logic;  -- pixel clock for syncing start of serial 
	           CLK_175 : in std_logic;  -- (x7) 175  MHz clock
	           de : in STD_LOGIC;  -- data enable (blank_bar)
	           RAM_DATA_RED: in std_logic_vector(5 downto 0);  -- Parallel RAM data
	           RAM_DATA_GREEN: in std_logic_vector(5 downto 0);  -- Parallel RAM data
	           RAM_DATA_BLUE: in std_logic_vector(5 downto 0);  -- Parallel RAM data
	           serial_out_1: out std_logic;  --Serial data to the LCD
	           serial_out_2: out std_logic;  --Serial data to the LCD
	           serial_out_3: out std_logic;  --Serial data to the LCD
	           serial_out_clk: out std_logic  --Serial data to the LCD
       );
   end component;
	
	component FSM is
		Port (RS0 			: in  STD_LOGIC;
				RS1 			: in  STD_LOGIC;
				RS2			: in	STD_LOGIC;
  				WR_bar 		: in  STD_LOGIC;
				clk			: in  STD_LOGIC;
				reset 		: in  STD_LOGIC;
				Load_Pix_Reg 	: out  STD_LOGIC;
				Load_Add_Reg 	: out  STD_LOGIC;
				Load_Rouge_Reg : out  STD_LOGIC;
				Load_vert_Reg 	: out  STD_LOGIC;
				Load_Bleu_Reg 	: out  STD_LOGIC;
				WR_bar_Ram 		: out  STD_LOGIC;
				M_RS2				: out	 STD_LOGIC;
				Set_Mux			: out	 STD_LOGIC;
				current_state, next_state : inout STD_LOGIC_VECTOR (3 downto 0)
		);
		
	end component;
	
	component LCD_Backlight is
	   port(
	     clk_out_25M : in STD_LOGIC;
         brightness_in : in STD_LOGIC_VECTOR ( 2 downto 0);
         pwm_out : out STD_LOGIC;
         led_on : out STD_LOGIC
         );
   end component;



-- dclaration des signaux entre les composants internes
    signal clk_175 : STD_LOGIC;
	signal clk_25  : STD_LOGIC;
	signal locked : STD_LOGIC;
	signal reset : STD_LOGIC;
	signal sig_WR_bar_Ram, sig_set_overlay, sig_A5, sig_Set_Mux, sig_M_RS2 : STD_LOGIC;
	signal sig_pixel_mask, sig_P0_P4_mask, sig_add_reg : STD_LOGIC_VECTOR (7 downto 0);
	signal sig_Ram_add : STD_LOGIC_VECTOR (7 downto 0);
	signal sig_Rouge_Reg, sig_vert_Reg, sig_Bleu_reg : STD_LOGIC_VECTOR (7 downto 0);
	signal sig_Load_Pix_Reg, sig_Load_Add_Reg : STD_LOGIC;
	signal sig_Load_Rouge_Reg, sig_Load_Vert_Reg, sig_Load_Bleu_Reg : STD_LOGIC; 
	signal sig_sel_0 : STD_LOGIC; 
	signal sig_A5_debug,sig_sel_0_debug :STD_LOGIC; 
	signal Rouge_LCD,Vert_LCD,Bleu_LCD : STD_LOGIC_VECTOR (7 downto 0); 
	
	
begin

-- interconnection des composants

    clock_generator : clk_wiz_0
       port map ( 
      -- Clock out ports  
       clk_out1 => clk_175,
       clk_out2 => clk_25,
      -- Status and control signals                1
       locked => locked,
       -- Clock in ports
       clk_in1 => clk
     );
     
     RST :reset_generator 
     port map (
     clock => clk_25,
     reset => reset
     );

	
	REG_P: REG_8 port map (
		d_in				=>		Data,
		d_out				=>		sig_pixel_mask,
		raz				=>		reset,		
		clk				=>		clk_25,
		load				=>		sig_Load_Pix_Reg 		
	);

	REG_A: REG_8 port map (
		d_in				=>		Data,
		d_out				=>		sig_add_Reg,
		raz				=>		reset,
		clk				=>		clk_25,
		load				=>		sig_Load_Add_Reg
	);

	REG_R: REG_8 port map (
		d_in				=>		Data,
		d_out				=>		sig_Rouge_Reg,
		raz				=>		reset,
		clk				=>		clk_25,
		load 				=>		sig_Load_Rouge_Reg
	);

	REG_V: REG_8 port map (
		d_in				=>		Data,
		d_out				=>		sig_vert_Reg,
		raz				=>		reset,
		clk				=>		clk_25,
		load				=>		sig_Load_vert_Reg
	);

	REG_B: REG_8 port map (
		d_in				=>		Data,
		d_out				=>		sig_Bleu_reg,
		raz				=>		reset,
		clk				=>		clk_25,
		load				=>		sig_Load_Bleu_Reg
	);
	
	PM: PIX_MASK port map (
		IN1 (4 downto 0) => 	palette, -- ATTENTION P4 de palette est  1 dans l'oscilloscop, il faudra le relier physiquement sur la carte !!!!
		IN1 (7 downto 5) => "000",
		IN2 				 =>	sig_pixel_mask,
		SORTIE 			=> 	sig_P0_P4_mask
	);

	M4: MUX_4 port map (
		E0 				=> 	sig_add_reg,
		E1 				=> 	sig_add_reg,
		E2					=> 	sig_P0_P4_mask,		
		E3	(3 downto 0)=>		Overlay, E3 (7 downto 4) => "0000",
		
		sel(1)			=> 	sig_Set_Mux,
		sel(0)			=>		sig_sel_0_debug, --**********************************************debug sig_sel_0 devient sig_sel_0_debug***************************************
		Sortie (7 downto 0)	=> sig_Ram_add

	--	Sortie (4 downto 0)	=>		sig_Ram_add (4 downto 0), 
	--	Sortie (7 downto 5)	=> 	OPEN	
	);
	
	

	RAM_R : RAM_VIDEO port map (
		RAM_ADDR(4 downto 0)	=> sig_Ram_add (4 downto 0),
		RAM_ADDR(5)		=>		sig_A5,
		RAM_WR			=>		sig_WR_bar_Ram,
		RAM_clk			=>		clk_25,
		RAM_DATA_IN		=>		sig_Rouge_Reg,
		RAM_DATA_OUT	=>		Rouge_LCD		

	);

	RAM_V : RAM_VIDEO port map (
		RAM_ADDR(4 downto 0)	=> sig_Ram_add (4 downto 0),
		RAM_ADDR(5)		=>		sig_A5,
		RAM_WR			=>		sig_WR_bar_Ram,
		RAM_clk			=>		clk_25,
		RAM_DATA_IN		=>		sig_vert_Reg,
		RAM_DATA_OUT	=>		Vert_LCD					

	);

	RAM_B : RAM_VIDEO port map (
		RAM_ADDR(4 downto 0)	=> sig_Ram_add (4 downto 0),
		RAM_ADDR(5)		=>		sig_A5,
		RAM_WR			=>		sig_WR_bar_Ram,
		RAM_clk			=>		clk_25,
		RAM_DATA_IN		=>		sig_Bleu_reg,
		RAM_DATA_OUT	=>		Bleu_LCD			
 
	);
	
	P2S : serializer port map  (
	       CLK_25 => clk_25,
           CLK_175 => clk_175,  -- (x7) 175  MHz clock
           de => blank_bar ,
           RAM_DATA_RED => Rouge_LCD (5 downto 0),   -- Parallel RAM data
           RAM_DATA_GREEN => Vert_LCD (5 downto 0),  -- Parallel RAM data
           RAM_DATA_BLUE => Bleu_LCD (5 downto 0),  -- Parallel RAM data
           serial_out_1 => LCD_Out1,   --Serial data to the LCD
           serial_out_2 => LCD_Out2, --Serial data to the LCD
           serial_out_3 =>  LCD_Out3, --Serial data to the LCD
           serial_out_clk => LCD_Clock_Out  --Serial data to the LCD
       );
       
    LCD_PWM : LCD_Backlight port map (
         clk_out_25M => clk_25,
         brightness_in => BL,
         pwm_out => PWM,
         led_on => LED_ON
         );
	
	FSM_1	: FSM 	port map (
		RS0				=>		RS0,
		RS1				=>		RS1,
		RS2				=>		RS2,
		WR_bar			=>		WR_bar,
		clk				=>		clk_25,
		reset				=>		reset,
		Load_Pix_Reg 	=>		sig_Load_Pix_Reg,
		Load_Add_Reg	=>		sig_Load_Add_Reg,
		Load_Rouge_Reg	=>		sig_Load_Rouge_Reg,
		Load_vert_Reg	=>		sig_Load_vert_Reg,
		Load_Bleu_Reg	=>		sig_Load_Bleu_Reg,
		WR_bar_Ram		=>		sig_WR_bar_Ram,
		Set_Mux			=>		sig_Set_Mux,
		M_RS2				=>		sig_M_RS2,
		current_state  =>		current_state,
		next_state		=>		OPEN
	);
	
	
	led1 <= locked;
	-- recopie de blank et horloge
	 --LCD_clock <= clk;
	 --LCD_blank_bar	<= blank_bar;
	 
	-- En mode ecriture RAM un pixel Overlay ne doit pas faire commuter le MUX
	
	Sig_sel_0 <= sig_Set_Mux and sig_set_overlay;
	
	
	-- detection de donnes overlay et adressage palette secteur RAM overlay
	
	sig_set_overlay <= (overlay (3)or overlay (2) or overlay (1) or overlay (0));
	sig_A5_debug <=   sig_sel_0  when sig_Set_Mux = '1' else
							sig_M_RS2; --****************************************Debug*****sig_A5 chang en sig_A5_debug***************************************
	
	
	-- pour le TB
	RAM_ADDR_bench (4 downto 0) <= sig_Ram_add (4 downto 0);
	RAM_ADDR_bench (5) <= sig_A5;
	RAM_ADDR_bench (7 downto 6) <= "00"; 
	
	-- ******************************************************* pour debug --***********************************
	
	Debug_out <= "0000";
	sig_A5 <= debug_in(1) or (debug_in(0) and sig_A5_debug);	-- quand jumper D0 ouvert force l'adressage de la memoire en palette. quand jumper D1 ferm force en overlay
	sig_sel_0_debug <= debug_in(2) or sig_sel_0; 				-- quand jumper en place force  les info overlay  passer :  l'ecran que les traces
	 
	
	
end Behavioral;



