
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
USE ieee.numeric_std.ALL;

-- A 64x8 single-port RAM in VHDL
entity RAM_VIDEO is
port(
 RAM_ADDR: in std_logic_vector(5 downto 0); 	-- Address to write/read RAM
 RAM_DATA_IN: in std_logic_vector(7 downto 0);  -- Data to write into RAM
 RAM_WR: in std_logic; -- Write enable 
 RAM_CLK: in std_logic; -- clk input for RAM
 RAM_DATA_OUT: out std_logic_vector(7 downto 0) -- Data output of RAM
);
end RAM_VIDEO;

architecture Behavioral of RAM_VIDEO is
-- define the new type for the 64x8 RAM 
type RAM_ARRAY is array (0 to 63 ) of std_logic_vector (7 downto 0);
-- initial values in the RAM
signal RAM: RAM_ARRAY :=(
   x"00",x"00",x"00",x"00",-- 0x00: Overlay DEPENDS OF THE JUMPER POSITION
   x"00",x"00",x"00",x"00",-- 0x04: Overlay
   x"00",x"00",x"00",x"00",-- 0x08: Overlay
   x"00",x"00",x"00",x"00",-- 0x0C: Overlay
   x"00",x"00",x"00",x"00",-- 0x10: Palette
   x"00",x"00",x"00",x"00",-- 0x14: Palette
   x"00",x"00",x"00",x"00",-- 0x18: Palette
   x"00",x"00",x"00",x"00",-- 0x1C: Palette
   x"00",x"00",x"00",x"00",-- 0x20: Overlay
   x"00",x"00",x"00",x"00",-- 0x24: Overlay
   x"00",x"00",x"00",x"00",-- 0x28: Overlay
   x"00",x"00",x"00",x"00",-- 0x2C: Overlay
   x"00",x"00",x"00",x"00",-- 0x30: 
   x"00",x"00",x"00",x"00",-- 0x34: 
   x"00",x"00",x"00",x"00",-- 0x38: 
   x"00",x"00",x"00",x"00" -- 0x3C: 
  
   ); 
begin
process(RAM_clk)
begin
 if(rising_edge(RAM_clk)) then
	if(RAM_WR='0') then -- when write enable = 0, 
	-- write input data into RAM at the provided address
		RAM(to_integer(unsigned(RAM_ADDR))) <= RAM_DATA_IN;
	-- The index of the RAM array type needs to be integer so
	-- converts RAM_ADDR from std_logic_vector -> Unsigned -> Interger using numeric_std library
	end if;

 end if;
end process;
 -- Data to be read out 
 RAM_DATA_OUT <= RAM(to_integer(unsigned(RAM_ADDR)));
end Behavioral;

