----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date: 10/10/2023 07:19:58 AM
-- Design Name: 
-- Module Name: Serializer - Behavioral
-- Project Name: RAM_DAC
-- Target Devices: 
-- Tool Versions: 
-- Description: Converts three 8 bit color vectors into three serial data streams to drive LVDS LCD inputs
-- 
-- Dependencies: 
-- 
-- Revision:
-- Revision 0.01 - File Created
-- Additional Comments:
-- 
----------------------------------------------------------------------------------


library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx leaf cells in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity Serializer is
port(
	   CLK_25 : in STD_LOGIC;
	   CLK_175 : in std_logic;  -- (x7) 175  MHz clock
	   de : in STD_LOGIC; -- data enable (blank_bar)
       RAM_DATA_RED: in std_logic_vector(5 downto 0);  -- Parallel RAM data
       RAM_DATA_GREEN: in std_logic_vector(5 downto 0);  -- Parallel RAM data
       RAM_DATA_BLUE: in std_logic_vector(5 downto 0);  -- Parallel RAM data
       serial_out_1: out std_logic;  --Serial data to the LCD
       serial_out_2: out std_logic;  --Serial data to the LCD
       serial_out_3: out std_logic;  --Serial data to the LCD
       serial_out_clk: out std_logic  --Serial data to the LCD
       );
end Serializer;

architecture Behavioral of Serializer is
signal red, grn, blu : std_logic_vector(5 downto 0);
signal de_latched : std_logic;

begin
    process (CLK_175, CLK_25)  
    variable lvds_count: natural range 0 to 7:= 0;
    variable synced: bit := '0';
        begin
            if rising_edge(CLK_175) then
                if CLK_25 = '1' and synced = '0' then
                    synced := '1';
                    red <= RAM_DATA_RED;
                    grn <= RAM_DATA_GREEN;
                    blu <= RAM_DATA_BLUE;
                    de_latched <= de;
                    lvds_count := 0; 
                else
                    if lvds_count < 6 then
                        lvds_count := lvds_count +1;  
                    end if;
                end if;
                
                case (lvds_count) is
                when 0 =>
                    serial_out_1 <= grn(0);
                    serial_out_2 <= blu(1);
                    serial_out_3 <= de_latched;
                    serial_out_clk <= '1';
               when 1 =>
                    serial_out_1 <= red(5);
                    serial_out_2 <= blu(0);
                    serial_out_3 <= '0';
                    serial_out_clk <= '1';
              when 2 =>
                    serial_out_1 <= red(4);
                    serial_out_2 <= grn(5);
                    serial_out_3 <= '0';
                    serial_out_clk <= '0';
              when 3 =>
                    serial_out_1 <= red(3);
                    serial_out_2 <= grn(4);
                    serial_out_3 <= blu(5);
                    serial_out_clk <= '0';
              when 4 =>
                    serial_out_1 <= red(2);
                    serial_out_2 <= grn(3);
                    serial_out_3 <= blu(4);
                    serial_out_clk <= '0';
              when 5 =>
                    serial_out_1 <= red(1);
                    serial_out_2 <= grn(2);
                    serial_out_3 <= blu(3);
                    serial_out_clk <= '1';
                    synced := '0';
              when 6 =>
                    serial_out_1 <= red(0);
                    serial_out_2 <= grn(1);
                    serial_out_3 <= blu(2);
                    serial_out_clk <= '1';
                when others =>   
                    serial_out_1 <= red(0);
                    serial_out_2 <= grn(1);
                    serial_out_3 <= blu(2);
                    serial_out_clk <= '1';
            end case;
            
        end if;
    end process;

end Behavioral;